/************************************************************************
     File:        TargaImage.h

     Author:     
                  Steven J. Chenney, schenney@cs.wisc.edu
     Modifier
                  Yu-Chi Lai, yu-chi@cs.wisc.edu
     
     Comment:     Class to handle the targa image
   
     Platform:    Visio Studio.Net 2003
*************************************************************************/

#ifndef TARGA_IMAGE_H_
#define TARGA_IMAGE_H_

#include <Fl/Fl.h>
#include <Fl/Fl_Widget.h>
#include <stdio.h>
#include <math.h>

class Color {
	public:
		float r, g, b, a;

		Color(float _r=0, float _g=0, float _b=0, float _a=1):r(_r),g(_g),b(_b),a(_a){};
		//void setRGB(float _r=0, float _g=0, float _b=0, float _a=0):r(_r),g(_g),b(_b),a(_a){};
		Color normalize() {
			float sum = r + g + b;
			return Color(r/sum, g/sum, b/sum, a);
		}

		//operators
		float& operator[](int value) {
			switch(value){
				case 0: 
					return r; break;
				case 1:
					return g; break;
				case 2:
					return b; break;
				case 3:
					return a; break;
				default: 
					printf("Color: Value out of range(%d)", value);
					//exit(1);
			}
		}
		
		const Color operator-(const Color & c){
			return Color(r - c.r, g - c.g, b - c.b, a - c.a);
		}
		
		const Color operator+(const Color & c){
			return Color(r + c.r, g + c.g, b + c.b, a + c.a);
		}

		Color& operator=(const Color & c){
			r = c.r; g = c.g; b = c.b; a = c.a;
			return *this;
		}

		float dif(const Color c){
			Color dif(r -c.r, g - c.g, b - c.b, a - c.a);
			return abs(dif.r) + abs(dif.g) + abs(dif.b);
		}
};

class TargaImage {
	// methods
	public:
		// Constructor and destructor
		TargaImage(void);
		TargaImage(int, int, unsigned char*);
		~TargaImage(void);

		// Convert the image to RGB format,
		unsigned char*	To_RGB(void);	   
		
		// Save the image to a file
		bool Save_Image(char*);    

		// Load a file and return a pointer to a new TargaImage object.  Returns NULL on failure
		static TargaImage* Load_Image(char*);
		static TargaImage* Copy_ESCAPI_Buffer(int * buffer, int width, int height);
		
		//Pixel functions
		unsigned char* getPixel(int x, int y);
		void setPixel(int x, int y, const unsigned char * pxl);
		void setPixel(int x, int y, Color c);
		void setPixel(int x, int y, int r=0, int g=0, int b=0, int a=1);

	private:
		int pos(int x, int y, int num_col);
		// helper function for format conversion
		void RGBA_To_RGB(unsigned char *data, unsigned char *rgb);          
		// reverse the rows of the image, some targas are stored bottom to top
		TargaImage* Reverse_Rows(void);
		

	// members
	public:
		int		         width;	    // width of the image in pixels
		int		         height;	    // height of the image in pixels
		unsigned char	*data;	    // pixel data for the image, assumed to be in pre-multiplied RGBA format.
		
		TargaImage& operator=(const TargaImage & img);	
};


#endif


