#include "binary.h"

/* returns position of <x>,<y>'th element in a single dimensional array
 * with <num_col> columns 
*/
int BinaryFilter::pos(int x, int y, int num_col){
	return y * num_col + x;
}

/*constructs a new filter of size <_width>, <_height> with <_filter_data>
*/
BinaryFilter::BinaryFilter(int _width, int _height, int * _filter_data){
	width = _width;
	height = _height;
	filter_data = _filter_data;
}

/*destructor
*/
BinaryFilter::~BinaryFilter(){
  if (filter_data)
      delete[] filter_data;
}

/* returns value of filter at position <x>, <y> 
*/
int BinaryFilter::getValue(int x, int y){
	if(x < 0 || x >= width || y < 0 || y >= height) {
		return NULL;
	} else {
		return *(filter_data + pos(x, y, width));
	}
}

/* returns false if pixel <x>, <y> of <img> 
 * is <c> connected and true otherwise
 */
bool BinaryFilter::erodePixel(TargaImage *img, int x, int y, int c){
	int radius = width/2;
	unsigned char * pxl;

	for(int i = -radius; i <= radius && c > 0; i++){
		for(int j = -radius; j <= radius && c > 0; j++){
			if((pxl = img->getPixel(x + i, y + j)) != NULL){
				if(getValue(i + radius, j + radius) * pxl[0] > 0) c--;
			}
		}
	}
	return (c > 0);
}

bool BinaryFilter::dilatePixel(TargaImage *img, int x, int y, int c){
	//return erodePixel(img, x, y, c);

	int radius = width/2;
	unsigned char * pxl = img->getPixel(x, y);
	if(pxl[0] != 0) return true;
	for(int i = -radius; i <= radius && c > 0; i++){
		for(int j = -radius; j <= radius && c > 0; j++){
			if(i == 0 && j == 0) continue;
			if((pxl = img->getPixel(x + i, y + j)) != NULL){
				if(getValue(i  + radius, j + radius) * pxl[0] > 0) c--;
			}
		}
	}
	return (c < 1);
}

TargaImage* erode(TargaImage* img, BinaryFilter* filter, int c){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * e_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	
	bool erode;
	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			if(erode = filter->erodePixel(img, x, y, c)){
				e_image->setPixel(x, y, 0, 0, 0);
			} else {
				e_image->setPixel(x, y, img->getPixel(x, y));
			}
		}
	}
	return e_image;
}

TargaImage* dilate(TargaImage* img, BinaryFilter* filter, int c){
	unsigned char* new_data = new unsigned char[img->width * img->height * 4];
	TargaImage * d_image = new TargaImage(img->width, img->height, new_data);
	delete[] new_data;
	
	bool dilate;
	for(int x = 0; x < img->width; x++){
		for(int y = 0; y < img->height; y++){
			if(dilate = filter->dilatePixel(img, x, y, c)){
				d_image->setPixel(x, y, 255, 255, 255);
			} else {
				d_image->setPixel(x, y, 0, 0, 0);
			}
		}
	}
	return d_image;
}

TargaImage* open(TargaImage* img){
	return img;
}

TargaImage* close(TargaImage* img){
	return img;
}