#include "colorspace.h"

double ccmin(double x, double y, double z){
	return min(min(x, y), z);
}

double ccmax(double x, double y, double z){
	return max(max(x, y), z);
}

void RGBtoHSL(float r, float g, float b, double* h, double* s, double* l){
	double vmin, vmax, delta;
	double dr,dg,db;
	r /= 255;
	g /= 255;
	b /= 255;
	vmin = ccmin(r, g, b);
	vmax = ccmax(r, g, b);
	delta = vmax - vmin;
	*l = (vmax + vmin) / 2;

	if(delta == 0){
		*h = 0;
		*s = 0;
	} else {
		if (*l < 0.5){
			*s = delta /(vmax + vmin);
		} else {
			*s = delta /(2 - vmax - vmin);
		}

		dr = (((vmax - r) / 6) + (delta / 2) ) / delta;
		dg = (((vmax - g) / 6) + (delta / 2) ) / delta;
		db = (((vmax - b) / 6) + (delta / 2) ) / delta;

		if(r == vmax){
			*h = db - dg;
		} else if(g == vmax) {
			*h = (1/3.0) + dr - db;
		} else if(b == vmax) {
			*h = (2/3.0) + dg - dr;
		}

		if(*h < 0) *h += 1;
		if(*h > 1) *h -= 1;
	}
}

double HueToRGB(double v1, double v2, double vH ){
	if (vH < 0) vH += 1;
	if (vH > 1) vH -= 1;
	if((6 * vH) < 1) return (v1 + ( v2 - v1 ) * 6 * vH);
	if((2 * vH) < 1) return (v2);
	if((3 * vH) < 2) return (v1 + (v2 - v1) * ((2/3.0) - vH ) * 6);
	//printf("HERE");
	return (v1);
}

void HSLtoRGB(double h, double s, double l, float * r, float * g, float * b){
	double v1, v2;
	if(s == 0.0){			//HSL values = 0  1
		*r = l * 255;		//RGB results = 0  255
		*g = l * 255;
		*b = l * 255;
	} else {
		if(l < 0.5) {
			v2 = l * (1 + s);
		} else {
			v2 = (l + s) - (s * l);
		}
		
		v1 = 2 * l - v2;
		*r = 255 * HueToRGB(v1, v2, h + (1/3.0));
		*g = 255 * HueToRGB(v1, v2, h);
		*b = 255 * HueToRGB(v1, v2, h - (1/3.0));
		//printf("(%3.2f,%3.2f,%3.2f) ", *r, *g, *b);
	}
}

void RGBtoYCC(float r, float g, float b, double* y, double* cb, double* cr){
	*y = .257 * r + .504 * g + .098 * b + 16;
	*cb = .148 * r - .291 * g + .439 * b + 128;
	*cr = .439 * r - .368 * g - .071 * b + 128;

	//Y = 0.299(R  C) + C + 0.114(B  C)
	//Cb = 0.564(B  Y)
	//Cr = 0.713(R  Y)
}

void YCCtoRGB(float y, float cb, float cr, double* r, double* g, double* b){
	*r = y + 1.371 * (cr - 128);
	*g = y - 0.698 * (cr - 128) - 0.336 * (cb  - 128);
	*b = y + 1.732 * (cb - 128);
}

void invertRGB(float r, float g, float b, float* ir, float* ig, float *ib){
	*ir = 255 - r;
	*ig = 255 - g;
	*ib = 255 - b;
}