#include <windows.h>
#include "escapi.h"
#include <stdio.h>
#include <tchar.h>

countCaptureDevicesProc countCaptureDevices;
initCaptureProc initCapture;
deinitCaptureProc deinitCapture;
doCaptureProc doCapture;
isCaptureDoneProc isCaptureDone;
getCaptureDeviceNameProc getCaptureDeviceName;
ESCAPIDLLVersionProc ESCAPIDLLVersion;

/* Internal: initialize COM */
typedef void (*initCOMProc)();
initCOMProc initCOM;

int setupESCAPI()
{
  /* Load DLL dynamically */
  HMODULE capdll = LoadLibrary(_T("escapi.dll"));
  if (capdll == NULL)
    return 0;

  /* Fetch function entry points */
  countCaptureDevices = (countCaptureDevicesProc)GetProcAddress(capdll, (LPCSTR)"countCaptureDevices");
  //fprintf(stderr, "HERE\n");
  initCapture = (initCaptureProc)GetProcAddress(capdll, (LPCSTR)"initCapture");
  deinitCapture = (deinitCaptureProc)GetProcAddress(capdll, (LPCSTR)"deinitCapture");
  doCapture = (doCaptureProc)GetProcAddress(capdll, (LPCSTR)"doCapture");
  isCaptureDone = (isCaptureDoneProc)GetProcAddress(capdll, (LPCSTR)"isCaptureDone");
  initCOM = (initCOMProc)GetProcAddress(capdll, (LPCSTR)"initCOM");
  getCaptureDeviceName = (getCaptureDeviceNameProc)GetProcAddress(capdll, (LPCSTR)"getCaptureDeviceName");
  ESCAPIDLLVersion = (ESCAPIDLLVersionProc)GetProcAddress(capdll, (LPCSTR)"ESCAPIDLLVersion");

  /* Check that we got all the entry points */
  if (initCOM == NULL ||
      ESCAPIDLLVersion == NULL ||
      getCaptureDeviceName == NULL ||
      countCaptureDevices == NULL ||
      initCapture == NULL ||
      deinitCapture == NULL ||
      doCapture == NULL ||
      isCaptureDone == NULL)
      return 0;

  /* Verify DLL version */
  if (ESCAPIDLLVersion() != 0x200)
    return 0;

  /* Initialize COM.. */
  initCOM();

  /* and return the number of capture devices found. */
  return countCaptureDevices();
}

