/* "ogl", example of using OpenGL with ESCAPI to create a "funny mirror" effect */

#ifdef _MSC_VER
#define NOMINMAX
#include <windows.h> // needed to get GL stuff to work
//#include <time.h>
#endif
#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "escapi.h"
#include "graphics.h"
#include "TargaImage.h"
#include "binary.h"
#include "convolution.h"

struct SimpleCapParams capture;
int device;
BinaryFilter *n4, *n8, *n24;
ConvolutionFilter *box, *tent, *bspline;

const int WINDOW_WIDTH = 800;//640;
const int WINDOW_HEIGHT = 800;//480;
const int IMAGE_WIDTH = 512;//256;
const int IMAGE_HEIGHT = 512;//256;

void render()
{
	int tick = SDL_GetTicks();

	glBegin(GL_QUADS);
		glTexCoord2f(0, 0);
		glVertex2f(  0,   0);
		glTexCoord2f(0, 1);
		glVertex2f(  0, WINDOW_HEIGHT);
		glTexCoord2f(1, 1);
		glVertex2f(WINDOW_WIDTH, WINDOW_HEIGHT);
		glTexCoord2f(1, 0);
		glVertex2f(WINDOW_WIDTH,   0);
	glEnd();

	// Show what we've got
	SDL_GL_SwapBuffers();
	// Don't hog all CPU time
	SDL_Delay(1);
	TargaImage * img = 0;
	TargaImage * old_img = 0;
	
	if (isCaptureDone(device))
	{	
		delete img;
		img = TargaImage::Copy_ESCAPI_Buffer(capture.mTargetBuf, capture.mWidth, capture.mHeight);
		

		// Load up the new data
		//img = colorLimit(img);
		//img = bw(img);
		//img = moveGlow(img, old_img);
		//img = blob(img);

		//img = greyscale(img);
		//img = SnobelEdge(img);
		//img = bw(img);
		//img = Invert(img);

		//img = warp(img, &warp1_pxl, 0);
		//img = resize(img, 400, 400, &bspline_filter);
		//img = convolute(img, bspline);
		//img = convolute(img, bspline);
		//img = HSL(img, 0, 1, 1);

		//img = Lichtenstein(img);
		img = Cartoonify(img);
		//img = specBeGone(img);
		//img = impress(img, 2);
		//img = convolute(img, bspline);
		//img = convolute(img, bspline);
		//img = YCC(img);
		//img = erode(img, n8, 3);
		//img = erode(img, n8, 3);
		//img = dilate(img, n8, 2);
		//img = dilate(img, n24, 17);
		//img = erode(img, n8, 3);
		//img = erode(img, n8, 3);
		//img = dilate(img, n24, 17);
		//img = dilate(img, n24, 4);
		//img = dilate(img, n8, 3);
		//img = dilate(img, n8, 3);


		//glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,400,400,0,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid*)img->data);

		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,IMAGE_WIDTH,IMAGE_HEIGHT,0,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid*)img->data);
		//glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,256,0,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid*)capture.mTargetBuf);
		
		// ..and ask for more
		//delete old_img;
		doCapture(device);
	}
}

// Entry point
int main(int argc, char *argv[])
{
	device = setupESCAPI();
	if (device < 1)
	{
        fprintf(stderr, "Unable to init ESCAPI\n");
        exit(1);
	}

	// Use the second device if more than one found
	// (makes my personal test setup easier =)
	if (device > 1)
		device = 1;
	else
		device = 0;

	// Setup ESCAPI to capture a 256x256 image.
	capture.mWidth = IMAGE_WIDTH;
	capture.mHeight = IMAGE_HEIGHT;
	capture.mTargetBuf = new int[IMAGE_WIDTH*IMAGE_HEIGHT];
	initCapture(device,&capture);
	doCapture(device);

	// Initialize SDL's subsystems - in this case, only video.
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
	{
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}


	// Register SDL_Quit to be called at exit; makes sure things are
	// cleaned up when we quit.
	atexit(SDL_Quit);

    int width = 640;
    int height = 480;
    int bpp = 32;

    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// Add the SDL_FULLSCREEN flag if you want to run fullscreen.
    int flags = SDL_OPENGL;// | SDL_FULLSCREEN;

    if (SDL_SetVideoMode(WINDOW_WIDTH, WINDOW_HEIGHT, bpp, flags) == 0) 
    {
        fprintf( stderr, "Video mode set failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

	// Set up OpenGL..
    glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	// Set up the view matrix so that both X and Y coordinates are reversed.
	// OpenGL normally considers 0,0 to be the bottom left corder, so reversing
	// the Y axis makes sense if we want to think the top left to be the origo,
	// and reversing the X axis makes sense since this is supposed to be a "mirror".
	gluOrtho2D(WINDOW_WIDTH, 0, WINDOW_HEIGHT, 0);
	glColor3f(1,1,1);

	// Create our texture
	glEnable(GL_TEXTURE_2D);
    GLuint texname;
    glGenTextures(1,&texname);
    glBindTexture(GL_TEXTURE_2D,texname);

	// Load up some initial data
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,IMAGE_WIDTH,IMAGE_HEIGHT,0,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid*)capture.mTargetBuf);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // Linear Filtering
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // Linear Filtering

	//create binary filters
	//N4 filter
	int * n4_data = new int[9];
	n4_data[0] = 0;	n4_data[0] = 1;	n4_data[0] = 0;
	n4_data[0] = 1;	n4_data[0] = 1;	n4_data[0] = 1;
	n4_data[0] = 0;	n4_data[0] = 1;	n4_data[0] = 0;
	n4 = new BinaryFilter(3, 3, n4_data);

	//N8 filter
	int * n8_data = new int[9];
	for(int i = 0; i < 9; i++){
		n8_data[i] = 1;
	}
	n8 = new BinaryFilter(3, 3, n8_data);

	//N24 filter
	int * n24_data = new int[25];
	for(int i = 0; i < 25; i++){
		n24_data[i] = 1;
	}
	n24 = new BinaryFilter(5, 5, n24_data);

	box = new ConvolutionFilter(&box_filter, 3, 3);
	tent = new ConvolutionFilter(&tentr_filter, 3, 3);
	bspline = new ConvolutionFilter(&bspline_filter, 5, 5);

	// Main loop: loop forever.
	while (1)
	{
		// Render stuff
		render();

		// Poll for events, and handle the ones we care about.
		SDL_Event event;
		while (SDL_PollEvent(&event)) 
		{
			switch (event.type) 
			{
			case SDL_KEYUP:
				switch (event.key.keysym.sym)
				{
				case SDLK_ESCAPE:
					deinitCapture(device);
					SDL_Quit();
					return 0;
					break;
				}
				break;
			case SDL_QUIT:
				deinitCapture(device);
				SDL_Quit();
				return(0);
			}
		}
	}
	return 0;
}